package gui;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import gui.*;
import demo.*;

class SimWinMgr extends JDesktopPane
{
	Hashtable simWindows;
	Integer curSimWin;
	Integer nextWindowID;

	public SimWinMgr(String args[])
	{
		setOpaque(false);

		simWindows = new Hashtable();
		curSimWin = null;
		nextWindowID = new Integer(1);
	}

	void createSimulationWindow(Vector fileNames)
	{
		Rectangle rect = getBounds();
		int width = (int)(rect.getWidth());
		int height = (int)(rect.getHeight());
		int x = ((nextWindowID.intValue()-1)%5)*width/3/4;
		int y = ((nextWindowID.intValue()-1)%5)*height/3/4;
		width = (int)(width*2.0/3.0);
		height = (int)(height*2.0/3.0);
		rect.setLocation(x,y);
		rect.setSize(width,height);

		String configFile = new String("blastula.cpa");
		String cellProgramFile = new String("simple.sim");
		BufferedReader cellProgramFileStream;
		BufferedReader simFileStream;
		BufferedWriter outFile;
		BufferedWriter statFile;
		BufferedWriter errFile;
		Vector ioObjects = new Vector();

		try
		{
			cellProgramFileStream = new BufferedReader(new FileReader((String)fileNames.elementAt(0)));
			simFileStream = new BufferedReader(new FileReader((String)fileNames.elementAt(1)));
			if(fileNames.elementAt(2).equals(""))
			{
				outFile = new BufferedWriter(new PrintWriter(System.out));
			}
			else
			{
				outFile = new BufferedWriter(new FileWriter((String)fileNames.elementAt(2)));
			}
			if(fileNames.elementAt(3).equals(""))
			{
				statFile = new BufferedWriter(new PrintWriter(System.out));
			}
			else
			{
				statFile = new BufferedWriter(new FileWriter((String)fileNames.elementAt(3)));
			}
			if(fileNames.elementAt(4).equals(""))
			{
				errFile = new BufferedWriter(new PrintWriter(System.err));
			}
			else
			{
				errFile = new BufferedWriter(new FileWriter((String)fileNames.elementAt(4)));
			}
			ioObjects.add(cellProgramFileStream);
			ioObjects.add(simFileStream);
			ioObjects.add(outFile);
			ioObjects.add(statFile);
			ioObjects.add(errFile);
		}
		catch(FileNotFoundException e)
		{
			System.err.println("Unable to open file: "+e.toString());
			return;
		}
		catch(IOException e)
		{
			System.err.println("Unable to open file: "+e.toString());
			return;
		}

		SimulationWindow simWin = new SimulationWindow(nextWindowID, "Simulation", rect, this, ioObjects);
		add(simWin, JLayeredPane.DEFAULT_LAYER);
		simWin.setVisible(true);

		simWindows.put(new Integer(nextWindowID.intValue()), simWin);
		nextWindowID = new Integer(nextWindowID.intValue() + 1);
	}

	void setCurrentSimulationWindow(Integer windowID)
	{
		curSimWin = new Integer(windowID.intValue());
	}

	void closeCurrentSimulationWindow()
	{
		if(curSimWin != null)
		{
			SimulationWindow simWin = (SimulationWindow)simWindows.get(curSimWin);
			curSimWin = null;
			if(simWin != null)
			{
				try
				{
					simWin.setClosed(true);
				}
				catch (java.beans.PropertyVetoException e)
				{
				}
			}
		}
	}

	void deleteSimulationWindow(Integer windowID)
	{
		simWindows.remove(new Integer(windowID.intValue()));
	}

	void nextStep()
	{
		if(curSimWin != null)
		{
			SimulationWindow simWin = (SimulationWindow)simWindows.get(curSimWin);
			if(simWin != null)
			{
				simWin.nextStep();
			}
		}
	}

	boolean exit()
	{
		int retVal;
		retVal = JOptionPane.showConfirmDialog(getParent(), "Are you sure?", "Exit CellSim", JOptionPane.YES_NO_OPTION);
		if(retVal == JOptionPane.YES_OPTION)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
}
