package parsers.CPA;

import java.util.*;
import util.*;
import core.*;
import parsers.CPA.*;

class CPACellProgram extends CellProgram
{
	private boolean resolved;
	Hashtable StateTable;

	CPACellProgram()
	{
		StateTable = new Hashtable();
		resolved = false;
	}

	void addState (CellState p_state)
	{
		StateTable.put(p_state.getState(),p_state);
	}

	void resolveStates ()
	{
	}

	public void runCellProgramForOneStep(Cell cell)
	{
		CellState cs = (CellState)StateTable.get(cell.getState());
		if(cs != null)
		{
			Enumeration enum = cs.getCellTransitionList().getList().elements();
			while(enum.hasMoreElements())
			{
				CellTransition ct = (CellTransition) enum.nextElement();
				if(isValidTransition(cell,ct) == true)
				{
					performTransition(cell,ct);
					break;
				}
			}
		}
	}

	boolean isValidTransition(Cell cell, CellTransition ct)
	{
		boolean isValid = true;

		Enumeration enum = ct.getChemicalConditionList().getList().elements();
		while(enum.hasMoreElements())
		{
			ChemicalCondition cc = (ChemicalCondition) enum.nextElement();
			Vector argVector = new Vector();
			argVector.add(new String("Check Condition"));
			argVector.add(new ChemicalCondition(cc));
			Vector retVec = cell.provideService(argVector);
			boolean isConditionSatisfied = ( (Boolean)retVec.elementAt(0) ).booleanValue();
			if(!isConditionSatisfied)
			{
				isValid = false;
				break;
			}
		}

		return isValid;
	}

	void performTransition(Cell cell, CellTransition ct)
	{
		Vector argVector = new Vector();
		if(ct.isCellDivision)
		{
			argVector.add(new String("Cell Division"));
			argVector.add(ct.getCellDivision().getState1());
			argVector.add(ct.getCellDivision().getState2());
			argVector.add(ct.getCellDivision().getDivisionType());
		}
		else
		{
			argVector.add(new String("State Change"));
			argVector.add(ct.getCellStateChange().getState());
		}
		cell.provideService(argVector);
	}

	public Vector provideService(Vector args)
	{
		if(args.elementAt(0).equals("Get State Color"))
		{
			
			Vector argVector = new Vector();
			CellState cs = (CellState)StateTable.get((String)args.elementAt(1));
			Color c = cs.getColor();
			if(c == null)
			{
				c = new Color(0.5,0.5,0.5);
			}
			argVector.add(c);
			return argVector;
		}
		return null;
	}

	public void init(Vector args)
	{
	}

	public String toString()
	{
		StringBuffer retVal = new StringBuffer();

		Enumeration enum = StateTable.elements();
		while(enum.hasMoreElements())
		{
			retVal.append( ( (CellState)enum.nextElement() ).toString() );
		}
		return retVal.toString();
	}
}
